local VERSION = tonumber(("$Revision: 42462 $"):match("%d+"))

Parrot = AceLibrary("AceAddon-2.0"):new("AceDB-2.0", "AceEvent-2.0", "AceConsole-2.0", "FuBarPlugin-2.0", "Parser-3.0", "AceModuleCore-2.0")
local Parrot, self = Parrot, Parrot
Parrot.version = "r" .. VERSION
Parrot.revision = VERSION
Parrot.date = ("$Date: 2007-07-02 00:38:22 -0400 (Mon, 02 Jul 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")

local waterfall = AceLibrary:HasInstance("Waterfall-1.0") and AceLibrary("Waterfall-1.0")

local _G = _G

local localeTables = {}
function Parrot:L(name)
	if not localeTables[name] then
		localeTables[name] = setmetatable({}, {__index = function(self, key)
			self[key] = key
			return key
		end})
	end
	return localeTables[name]
end

local L = Parrot:L("Parrot")

local SharedMedia = AceLibrary("SharedMedia-1.0")

Parrot:RegisterDB("ParrotDB")
Parrot:RegisterDefaults('profile', {
	font = "Friz Quadrata TT",
	fontSize = 20,
})

-- FuBar configuration
Parrot.hasIcon = [[Interface\Icons\Spell_Nature_ForceOfNature]]
Parrot.hideWithoutStandby = true
Parrot.defaultPosition = "MINIMAP"
Parrot.defaultMinimapPosition = 250
Parrot.hasNoText = true
Parrot.hasNoColor = true
Parrot.cannotDetachTooltip = true
Parrot.independentProfile = true
Parrot.blizzardTooltip = true

function Parrot:OnInitialize()
	self:RegisterChatCommand('/Parrot', '/Par', {
		type = 'execute',
		func = function()
			self:OpenMenu()
		end
	})
end

local ParrotFrame

function Parrot:OnEnable()
	if not ParrotFrame then
		ParrotFrame = CreateFrame("Frame", "ParrotFrame", UIParent)
		ParrotFrame:SetFrameStrata("HIGH")
		ParrotFrame:SetToplevel(true)
		ParrotFrame:SetPoint("CENTER")
		ParrotFrame:SetWidth(0.0001)
		ParrotFrame:SetHeight(0.0001)
	end
end

function Parrot:OnDisable()
end

function Parrot:OnClick()
	if waterfall then
		waterfall:Open("Parrot")
	else
		self:Print(L["Waterfall-1.0 is required to access the GUI."])
	end
end

local animationStyles = {
}
local animationStylesCleanup = {
}
local animationStylesValidate = {
}
function Parrot:RegisterAnimationStyle(name, localName, func, cleanup)
	AceLibrary.argCheck(self, name, 2, "string")
	AceLibrary.argCheck(self, localName, 3, "string")
	AceLibrary.argCheck(self, func, 4, "function")
	AceLibrary.argCheck(self, cleanup, 5, "function", "nil")
	if animationStyles[name] then
		error(("Animation style %q already registered"):format(name), 2)
	end
	animationStyles[name] = func
	animationStylesValidate[name] = localName
	animationStylesCleanup[name] = cleanup
end

local scrollAreas = {
	["Notification"] = {
		align = "CENTER",
		animationStyle = "Straight",
		reverse = false,
		height = 150,
		xOffset = 0,
		yOffset = 100,
	},
}

local freeFontStrings = {}
local wildFontStrings = {}
local fontString_num = 0
function Parrot:ShowMessage(text, scrollArea)
	if not self:IsActive() then
		return
	end
	if not scrollAreas[scrollArea] then
		if scrollAreas["Notification"] then
			scrollArea = "Notification"
		else
			scrollArea = next(scrollAreas)
			if not scrollArea then
				return
			end
		end
	end
	local fs = next(freeFontStrings)
	if fs then
		fs:Show()
		fs:SetAlpha(1)
		fs:ClearAllPoints()
		freeFontStrings[fs] = nil
	else
		fontString_num = fontString_num + 1
		fs = ParrotFrame:CreateFontString("ParrotFrameFontString" .. fontString_num, "ARTWORK", "MasterFont")
	end
	
	fs:SetFont(SharedMedia:Fetch('font', self.db.profile.font), self.db.profile.fontSize)
	fs:SetText(text)
	fs.start = GetTime()
	fs.finish = GetTime()+3
	fs.length = 3
	fs.scrollArea = scrollAreas[scrollArea]
	fs:SetPoint("BOTTOM", 0, 0)
	wildFontStrings[fs] = true
	
	self:ScheduleRepeatingEvent("Parrot_OnUpdate", self.OnUpdate, 0, self)
end

function Parrot.modulePrototype:ShowMessage(text, position)
	Parrot:ShowMessage(text, position)
end

function Parrot:OnUpdate()
	local now = GetTime()
	for fs in pairs(wildFontStrings) do
		local start, length = fs.start, fs.length
		if start + length <= now then
			local scrollArea = fs.scrollArea
			if scrollArea then
				local animationStyle = scrollArea.animationStyle
				local cleanup
				if animationStyles[animationStyle] then
					cleanup = animationStylesCleanup[animationStyle]
				else
					cleanup = animationStylesCleanup.Straight
				end
				if cleanup then
					cleanup(fs, scrollArea)
				end
			end
			fs:Hide()
			wildFontStrings[fs] = nil
			freeFontStrings[fs] = true
		else
			local percent = (now - start) / length
			if percent >= 0.75 then
				fs:SetAlpha((1-percent) * 4)
			end
			
			local scrollArea = fs.scrollArea
			if scrollArea then
				local func = animationStyles[scrollArea.animationStyle]
				if not func then
					func = animationStyles.Straight
				end
				if func then
					func(fs, scrollArea, percent)
				end
			end
		end
	end
	if not next(wildFontStrings) then
		self:CancelScheduledEvent("Parrot_OnUpdate")
	end
end

Parrot.options = {
	type = 'group',
	handler = Parrot,
	args = {
	}
}

if waterfall then
	waterfall:Register("Parrot", 
						"aceOptions", Parrot.options,
						"title", L["Parrot Configuration"],
						"treeLevels", 5);
end

Parrot.OnMenuRequest = Parrot.options

function Parrot:OnTooltipUpdate()
	GameTooltip:AddLine("Parrot")
	GameTooltip:AddLine(" ")
	if waterfall then
		GameTooltip:AddLine(L["|cffffff00Left-Click|r to change settings with a nice GUI configuration."], 0.2, 1, 0.2)
	end
	GameTooltip:AddLine(L["|cffffff00Right-Click|r to change settings with a drop-down menu."], 0.2, 1, 0.2)
end
